@@ -214,12 +214,9 @@ setMethod("import.bw", "ANY",
 
 setMethod("import", "BigWigFile",
           function(con, format, text, selection = BigWigSelection(which, ...),
-                   which = con, asRangedData = FALSE, asRle = FALSE,
+                   which = con, asRle = FALSE,
                    as = c("GRanges", "RleList", "NumericList"), ...)
           {
-            if (asRangedData) {
-              stop("'asRangedData' argument is defunct")
-            }
             if (asRle) {
               warning("'asRle' argument has been deprecated, ",
                       "use 'as=\"RleList\"' instead")
@@ -229,20 +226,19 @@ setMethod("import", "BigWigFile",
               checkArgFormat(con, format)
             as <- match.arg(as)
 
-            if (as != "NumericList")
-                asRangedData <- normarg_asRangedData(asRangedData, "import")
             selection <- as(selection, "BigWigSelection")
             validObject(selection)
             si <- seqinfo(con)
             which <- ranges(selection)
-            badSpaces <- setdiff(names(which), seqnames(si))
-            if (length(badSpaces))
-              stop("'which' contains sequence names not known to BigWig file: ",
-                   paste(badSpaces, collapse = ", "))
+            badSpaces <- setdiff(names(which), seqlevels(si))
+            if (length(badSpaces) > 0L)
+              warning("'which' contains seqlevels not known to BigWig file: ",
+                      paste(badSpaces, collapse = ", "))
+            which <- which[names(which) %in% seqlevels(si)]
             flatWhich <- unlist(which, use.names = FALSE)
             if (is.null(flatWhich))
               flatWhich <- IRanges()
-            which <- split(flatWhich, factor(space(which), seqnames(si)))
+            which <- split(flatWhich, factor(space(which), seqlevels(si)))
             if (as != "NumericList") {
               which <- as(which, "NormalIRangesList")
             }
