@@ -90,7 +90,7 @@ setMethod("show", "BrowserView", function(object)
                       unlist(end(range)), sep = ""),
                 "\n")
             nms <- paste("'", names(trackNames(object)), "'", sep = "")
-            cat(BiocGenerics:::labeledLine("trackNames", nms))
+            cat(S4Vectors:::labeledLine("trackNames", nms))
           })
 
 setClass("BrowserViewList", contains = "SimpleList",
@@ -198,10 +198,6 @@ setMethod("browserSession", "missing",
 # get one from a view
 setMethod("browserSession", "BrowserView", function(object) object@session)
 
-# load a sequence into the browser (probably should remove this)
-setGeneric("sequence<-", function(object, name, ..., value)
-           standardGeneric("sequence<-"))
-
 setGeneric("login",
            function(x, username, password, ...) standardGeneric("login"),
            signature = "x")
