@@ -165,7 +165,8 @@ setMethod("export", c("GenomicRanges", "GFFFile"),
                     x_char <- paste0("\"", x_char, "\"")
                   paste(name, x_char, sep = tvsep)
                 }, simplify = FALSE))
-                attrs <- do.call(paste, c(attrs, sep = "; "))
+                if (version == "3") sep <- ";" else sep <- "; "
+                attrs <- do.call(paste, c(attrs, sep = sep))
                 attrs <- gsub("[^;]*?\r\"?(;|$)", "", attrs)
                 attrs[nchar(attrs) == 0] <- NA
               }
@@ -219,81 +220,18 @@ setMethod("import.gff", "ANY",
             import(con, "gff", ...)
           })
 
-.parse_attrCol <- function(attrCol, file, colnames) {
-  if (is(file, "GFF1File")) {
-    if (is.null(colnames) || "group" %in% colnames)
-      attrList <- list(group = factor(attrCol, levels=unique(attrCol)))
-    else attrList <- list()
-  } else {
-    if (length(attrCol) == 0L)
-        return(list())
-    sentinel <- "\b"
-    con <- file()
-    on.exit(close(con))
-    writeLines(paste0(sub("; *$", "", attrCol), sentinel), con)
-    attrs <- scan(con, what=character(),
-                  strip.white = TRUE, quiet = TRUE, sep = ";",
-                  quote = if (is(file, "GFF3File")) "" else "\"")
-    lines <- togroup(PartitioningByEnd(grep(sentinel, attrs, fixed = TRUE)))
-    attrs <- sub(sentinel, "", attrs, fixed = TRUE)
-    tag.value.sep <- if (is(file, "GFF3File")) "=" else " "
-    sep.pos <- regexpr(tag.value.sep, attrs, fixed=TRUE)
-    if (any(sep.pos == -1L))
-      stop("Some attributes do not conform to 'tag", tag.value.sep,
-           "value' format")
-    tags <- substring(attrs, 1L, sep.pos - 1L)
-    vals <- substring(attrs, sep.pos + 1L, nchar(attrs))
-    if (!is.null(colnames)) {
-      keep <- tags %in% colnames
-      lines <- lines[keep]
-      vals <- vals[keep]
-      tags <- urlDecode(tags[keep])
-    }
-    tags <- factor(tags, levels=unique(tags))
-    lineByTag <- split(lines, tags)
-    valByTag <- split(vals, tags)
-
-    ## FIXME: Parent, Alias, Note, DBxref,
-    ## Ontology_term are allowed to have multiple
-    ## values. We should probably always return them as a
-    ## CharacterList.
-    multiTags <- c("Parent", "Alias", "Note", "DBxref", "Ontology_term")
-    attrList <- sapply(levels(tags), function(tagName) {
-      vals <- valByTag[[tagName]]
-      if (is(file, "GFF3File") &&
-          (any(grepl(",", vals, fixed=TRUE)) || tagName %in% multiTags)) {
-        vals <- CharacterList(strsplit(vals, ",", fixed=TRUE))
-        vals <- relist(urlDecode(unlist(vals)), vals)
-        coerced <- suppressWarnings(as(vals, "NumericList"))
-        if (!any(any(is.na(coerced))))
-          vals <- coerced
-        vec <- as(rep.int(list(character()), length(attrCol)), class(vals))
-      } else {
-        coerced <- suppressWarnings(as.numeric(vals))
-        if (!any(is.na(coerced)))
-          vals <- coerced
-        if (is(file, "GFF3File"))
-          vals <- urlDecode(vals)
-        vec <- rep.int(NA, length(attrCol))
-      }
-      vec[lineByTag[[tagName]]] <- vals
-      vec
-    }, simplify = FALSE)
-  }
-  attrList
-}
-
 setMethod("import", "GFFFile",
           function(con, format, text, version = c("", "1", "2", "3"),
-                   genome = NA, asRangedData = FALSE, colnames = NULL,
-                   which = NULL, feature.type = NULL)
+                   genome = NA, colnames = NULL,
+                   which = NULL, feature.type = NULL,
+                   sequenceRegionsAsSeqinfo = FALSE)
           {
             if (!missing(format))
               checkArgFormat(con, format)
             if (!missing(version))
               con <- asGFFVersion(con, match.arg(version))
-            asRangedData <- normarg_asRangedData(asRangedData, "import")
-
+            stopifnot(isTRUEorFALSE(sequenceRegionsAsSeqinfo))
+            
             ## download the file first if it's remote
             if (is.character(resource(con))) {
                 uri <- .parseURI(resource(con))
@@ -304,7 +242,7 @@ setMethod("import", "GFFFile",
                 }
             }
 
-            sniffed <- sniffGFFVersion(resource(con))
+            sniffed <- .sniffGFFVersion(resource(con))
             version <- gffFileVersion(con)
             if (!length(version)) {
               if (is.null(sniffed))
@@ -317,83 +255,37 @@ setMethod("import", "GFFFile",
               warning("gff-version directive indicates version is ", sniffed,
                       ", not ", version)
 
-            if (is.na(genome))
+            if (is.na(genome)) {
               genome <- genome(con)
+              if (is.null(genome))
+                  genome <- NA
+            }
             
 ### FIXME: a queryForLines() function would be more efficient
-            file <- con
-            con <- queryForResource(con, which)
-            lines <- readLines(con, warn = FALSE) # unfortunately, not a table
-            lines <- lines[nzchar(lines)]
-            
-            ## strip comments
-            notComments <- which(substr(lines, start=1L, stop=1L) != "#")
-            lines <- lines[notComments]
-            
-### TODO: handle ontologies (store in RangedData)
 
-            ## strip FASTA sequence
-            fastaHeaders <- which(substr(lines, start=1L, stop=1L) == ">")
-            if (length(fastaHeaders))
-              lines <- head(lines, fastaHeaders[1] - 1)
+            ## Temporarily disable use of Tabix Index.
+            ## TODO: Restore use of Tabix Index!
+            #con <- queryForResource(con, which)
+            con <- queryForResource(con)
 
-            ## construct table
-            fields <- c("seqname", "source", "type", "start", "end", "score",
-                        "strand", "phase", "attributes")
-            linesSplit <- strsplit(lines, "\t", fixed=TRUE)
-            fieldCounts <- elementLengths(linesSplit)
-            if (any(fieldCounts > length(fields)) ||
-                any(fieldCounts < (length(fields) - 1)))
-              stop("GFF files must have ", length(fields),
-                   " tab-separated columns")
-            haveAttr <- fieldCounts == length(fields)
-            data <- unlist(linesSplit[haveAttr], use.names=FALSE)
-            if (is.null(data))
-                data <- character(0)
-            haveAttrMat <- matrix(data, ncol=length(fields), byrow=TRUE)
-            data <- unlist(linesSplit[!haveAttr], use.names=FALSE)
-            if (is.null(data))
-                data <- character(0)
-            noAttrMat <- matrix(data, ncol=length(fields)-1L, byrow=TRUE)
-            noAttrMat <- cbind(noAttrMat, rep.int("", nrow(noAttrMat)))
-            table <- rbind(noAttrMat, haveAttrMat)
-            colnames(table) <- fields
-
-            if (!is.null(feature.type))
-                table <- table[table[,"type"] %in% feature.type,,drop=FALSE]
-
-            ## handle missings
-            table[table == "."] <- NA_character_
-            
-            attrCol <- table[,"attributes"]
-            if (is(file, "GFF3File")) {
-              table <- table[,setdiff(colnames(table), "attributes"),drop=FALSE]
-              table[table[,"strand"] == "?","strand"] <- NA_character_
-              is_not_NA <- !is.na(table)
-              table[is_not_NA] <- urlDecode(table[is_not_NA])
+            gr <- readGFFAsGRanges(con,
+                                   version=version,
+                                   colnames=colnames,
+                                   filter=list(type=feature.type),
+                                   genome=genome,
+                                   sequenceRegionsAsSeqinfo=
+                                       sequenceRegionsAsSeqinfo,
+                                   speciesAsMetadata=TRUE)
+            if (is.na(genome)) {
+                ans_seqinfo <- seqinfo(gr)
+            } else {
+                ans_seqinfo <- NULL
             }
-            
-            extraCols <- c("source", "type", "score", "strand", "phase")
-            if (!is.null(colnames))
-              extraCols <- intersect(extraCols, colnames)
-            xd <- as(table[,extraCols,drop=FALSE], "DataFrame")
-
-            if (!is.null(xd$phase))
-              xd$phase <- as.integer(as.character(xd$phase))
-            if (!is.null(xd$score))
-              suppressWarnings(xd$score <- as.numeric(as.character(xd$score)))
-
-            if (is.null(colnames) || length(setdiff(colnames, extraCols))) {
-              attrList <- .parse_attrCol(attrCol, file, colnames)
-              xd <- DataFrame(xd, attrList)
-            }
-            
-            end <- as.integer(table[,"end"])
-            GenomicData(IRanges(as.integer(table[,"start"]), end),
-                        xd, chrom = table[,"seqname"], genome = genome,
-                        seqinfo = attr(con, "seqinfo"),
-                        asRangedData = asRangedData,
-                        which = if (attr(con, "usedWhich")) NULL else which)
+            GenomicData(ranges(gr), mcols(gr),
+                        strand = strand(gr), chrom = seqnames(gr),
+                        genome = genome, seqinfo = ans_seqinfo,
+                        which = if (attr(con, "usedWhich")) NULL else which,
+                        metadata = metadata(gr))
           })
 
 setGeneric("import.gff1",
@@ -490,7 +382,7 @@ setMethod("genome", "GFFFile", function(x) providerVersion(x))
 gffComment <- function(con, ...) 
   cat("##", paste(...), "\n", sep = "", file = con, append = TRUE)
 
-sniffGFFVersion <- function(con) {
+.sniffGFFVersion <- function(con) {
   con <- connectionForResource(con, "r")
   version <- NULL
   lines <- line <- readLines(con, n = 1)
