@@ -46,6 +46,12 @@ setMethod("show", "RTLFile", function(object) {
 })
 
 FileForFormat <- function(path, format = file_ext(path)) {
+  if (!(isSingleString(path) || is(path, "connection")))
+    stop("'path' must be a single string or a connection object")
+  if (!isSingleString(format))
+    stop("'format' must be a single string")
+  if (format == "")
+    stop("Cannot detect format (no extension found in file name)")
   fileClassName <- paste0(format, "File")
   signatureClasses <- function(fun, pos) {
     matrix(unlist(findMethods(fun)@signatures), 3)[pos,]
@@ -59,9 +65,8 @@ FileForFormat <- function(path, format = file_ext(path)) {
   fileClassNames <- c(fileClassNames, fileSubClassNames) 
   fileClassIndex <- match(tolower(fileClassName),
                           tolower(fileClassNames))
-  if (is.na(fileClassIndex)) {
+  if (is.na(fileClassIndex))
     stop("Format '", format, "' unsupported")
-  }
   fileClassName <- fileClassNames[fileClassIndex]
   fileClass <- getClass(fileClassName)
   pkg <- packageSlot(fileClass)
@@ -185,12 +190,13 @@ setMethod("bestFileFormat", c("RangesList", "ANY"), function(x, dest) {
   "bed" # just ranges...
 })
 
-file_ext <- function(con) gsub(".*\\.([^.]*)$", "\\1", con)
-
 ## Uses XML::parseURI, except first checks for Windows drive letter.
 ## There are no known URI schemes that are only a single character.
 .parseURI <- function(uri) {
-  if (.Platform$OS.type == "windows" && grepl("^[A-Za-z]:[/\\]", uri)) {
+  windowsDriveLetter <- .Platform$OS.type == "windows" &&
+      grepl("^[A-Za-z]:[/\\]", uri)
+  hasScheme <- grepl("^[A-Za-z]+:", uri) && !windowsDriveLetter
+  if (!hasScheme) {
     parsed <- parseURI("")
     parsed$path <- uri
   } else {
@@ -274,7 +280,7 @@ manage <- function(con) {
     return(con)
   env <- new.env()
   finalizer <- function(obj) {
-    if (exists("con")) {
+    if (exists("con", parent.env(environment()), inherits=FALSE)) {
       close(con)
       rm(con, inherits = TRUE)
       TRUE
