@@ -17,9 +17,13 @@ setMethod("import", c("TabixFile", "character"),
               args <- list(...)
               if (is.null(args$comment.char))
                 args$comment.char <- tabixHeader$comment
-              do.call(import.trackTable,
+              skip <- tabixHeader$skip
+              if (header) {
+                skip <- skip - 1L
+              }
+              do.call(import.tabSeparated,
                       c(list(buffer), genome = genome, tabixHeader$indexColumns,
-                        args))
+                        skip = skip, header = header, args))
             } else import(file, genome = genome, ...)
           })
 
@@ -28,3 +32,14 @@ setMethod("import", c("TabixFile", "missing"),
           {
             import(con, file_ext(file_path_sans_ext(path(con))), ...)
           })
+
+setGeneric("exportToTabix",
+           function(object, con, ...) standardGeneric("exportToTabix"))
+
+setMethod("exportToTabix", c("ANY", "character"),
+          function(object, con, ...) {
+            con <- TabSeparatedFile(con)
+            export(sort(object, ignore.strand=TRUE), con,
+                   row.names=FALSE, col.names=TRUE, ...)
+            indexTrack(con, seq=1L, start=2L, end=3L, skip=1L)
+          })
