@@ -45,18 +45,6 @@ setMethod("export", c("ANY", "TwoBitFile"), function(object, con, format, ...) {
   callGeneric()
 })
 
-setMethod("export", c("BSgenome", "TwoBitFile"),
-          function(object, con, format, ...) {
-            if (!missing(format))
-              checkArgFormat(con, format)
-            i <- 0L
-            twoBits <- bsapply(new("BSParams", X = object, FUN = function(chr) {
-              i <<- i + 1
-              .DNAString_to_twoBit(chr, seqnames(object)[i])
-            }, ...))
-            invisible(.TwoBits_export(as.list(twoBits), twoBitPath(path(con))))
-          })
-
 setMethod("export", c("DNAStringSet", "TwoBitFile"),
           function(object, con, format) {
             if (!missing(format))
@@ -64,12 +52,22 @@ setMethod("export", c("DNAStringSet", "TwoBitFile"),
             seqnames <- names(object)
             if (is.null(seqnames))
               seqnames <- as.character(seq(length(object)))
+            freq <- alphabetFrequency(object)
+            unsupported.chars <- setdiff(DNA_ALPHABET, c(DNA_BASES, "N"))
+            if (any(rowSums(freq[,unsupported.chars,drop=FALSE]) > 0L)) {
+              stop("One or more strings contain unsupported ambiguity ",
+                   "characters.\nStrings can contain only A, C, G, T or N.")
+            }
+            if (any(width(object) == 0L)) {
+              stop("Empty strings are not yet supported")
+            }
             invisible(.TwoBits_export(mapply(.DNAString_to_twoBit, object,
                                              seqnames),
                                       twoBitPath(path(con))))
           })
 
-## Hidden export of a list of twoBit pointers
+## Hidden export of a list of twoBit pointers.
+## NOT exported (but used in the BSgenome package).
 .TwoBits_export <- function(object, con) {
   if (!isSingleString(con))
     stop("'con' must be a single, non-NA string")
@@ -79,6 +77,7 @@ setMethod("export", c("DNAStringSet", "TwoBitFile"),
   .Call(TwoBits_write, object, con)
 }
 
+## NOT exported (but used in the BSgenome package).
 .DNAString_to_twoBit <- function(object, seqname) {
   if (!isSingleString(seqname))
     stop("'seqname' must be a single, non-NA string")
