@@ -9,10 +9,11 @@ test_gff <- function() {
            rep(94852, 3), rep(95178, 2), 95504,
            rep(c(91263, 88017, 88392, 88771, 89018, 89827, 90655, 91263),
                each = 2))
-  type <- factor(c("gene", "mRNA", "mRNA", "exon", "CDS", "exon", "exon",
-                   "CDS", "CDS", "exon", "exon", "CDS", "exon", "CDS", "exon",
-                   "gene", "mRNA", "exon", "CDS", "exon", "CDS", "exon", "CDS",
-                   "exon", "CDS", "exon", "CDS", "exon", "CDS", "exon", "CDS"))
+  type <- c("gene", "mRNA", "mRNA", "exon", "CDS", "exon", "exon",
+            "CDS", "CDS", "exon", "exon", "CDS", "exon", "CDS", "exon",
+            "gene", "mRNA", "exon", "CDS", "exon", "CDS", "exon", "CDS",
+            "exon", "CDS", "exon", "CDS", "exon", "CDS", "exon", "CDS")
+  type <- factor(type, unique(type))
   source <- factor("rtracklayer")
   phase <- NA_integer_
   score <- c(5, rep(NA, length(type) - 1L))
@@ -59,8 +60,11 @@ test_gff <- function() {
   checkIdentical(correct_gff3, test)
   test <- import.gff3(test_gff3)
   checkIdentical(correct_gff3, test)
-  options(warn = 2)
+  #suppressWarnings(test <- import.gff2(test_gff3))
+  #checkIdentical(correct_gff3, test)
+  oldOpts <- options(warn = 2)
   checkException(import.gff2(test_gff3))
+  options(oldOpts)
   
   ## TEST: GFF(3)File
   test_gff_file <- GFF3File(test_gff3)
@@ -73,7 +77,11 @@ test_gff <- function() {
   test <- import(test_gff_file)
   checkIdentical(correct_gff3, test)
   test_gff_file <- GFF2File(test_gff3)
+  #suppressWarnings(test <- import(test_gff_file))
+  #checkIdentical(correct_gff3, test)
+  oldOpts <- options(warn = 2)
   checkException(test <- import(test_gff_file))
+  options(oldOpts)
 
   ## TEST: 'gff' extension
   test_gff_out <- file.path(tempdir(), "genes.gff")
@@ -96,8 +104,12 @@ test_gff <- function() {
   checkIdentical(test, correct_gff3)
   test <- import(test_gff_out, version = "3")
   checkIdentical(test, correct_gff3)
+  #suppressWarnings(test <- import(test_gff_out, version = "2"))
+  #checkIdentical(test, correct_gff3)
+  oldOpts <- options(warn = 2)
   checkException(test <- import(test_gff_out, version = "2"))
-  
+  options(oldOpts)
+
   ## TEST: 'gff2' extension
   test_gff2_out <- file.path(tempdir(), "genes.gff2")
   export(correct_gff3, test_gff2_out)
@@ -139,15 +151,11 @@ test_gff <- function() {
   ## TEST: colnames empty, colnames := "geneName", colnames := "strand"
   test <- import(test_gff3, colnames = character())
   target <- correct_gff3[,character()]
-  strand(target) <- "*"
   checkIdentical(target, test)
   test <- import(test_gff3, colnames = "geneName")
   target <- correct_gff3[,"geneName"]
-  strand(target) <- "*"
-  checkIdentical(target, test)
-  test <- import(test_gff3, colnames = "strand")
-  target <- correct_gff3[,character()]
   checkIdentical(target, test)
+  checkException(import(test_gff3, colnames = "strand"))
 
   ## TEST: import from connection
   test_gff_con <- file(test_gff_out)
