@@ -1,4 +1,5 @@
 #include "rtracklayer.h"
+#include "readGFF.h"
 #include "bigWig.h"
 #include "twoBit.h"
 #include "utils.h"
@@ -8,6 +9,11 @@
 #define CALLMETHOD_DEF(fun, numArgs) {#fun, (DL_FUNC) &fun, numArgs}
 
 static const R_CallMethodDef callMethods[] = {
+  /* readGFF.c */
+  CALLMETHOD_DEF(gff_colnames, 1),
+  CALLMETHOD_DEF(read_gff_pragmas, 1),
+  CALLMETHOD_DEF(scan_gff, 5),
+  CALLMETHOD_DEF(load_gff, 8),
   /* bigWig.c */
   CALLMETHOD_DEF(BWGSectionList_add, 5),
   CALLMETHOD_DEF(BWGSectionList_write, 4),
