@@ -3,17 +3,18 @@
  * This file is copyright 2002 Jim Kent, but license is hereby
  * granted for all use - public, private or commercial. */
 
+#include "common.h"
+#include <string.h>
+
 #ifndef WIN32
 
-#include "common.h"
 #include <signal.h>
 #include <errno.h>
-#include <string.h>
 #include <sys/time.h>
 #include <utime.h>
 #include <pthread.h>
 #include "internet.h"
-#include "errabort.h"
+#include "errAbort.h"
 #include "hash.h"
 #include "net.h"
 #include "linefile.h"
@@ -23,10 +24,7 @@
 #include "sqlNum.h"
 #include "obscure.h"
 
-static char const rcsid[] = "$Id: net.c,v 1.80 2010/04/14 07:42:06 galt Exp $";
-
 /* Brought errno in to get more useful error messages */
-
 extern int errno;
 
 static int netStreamSocket()
@@ -39,17 +37,79 @@ if (sd < 0)
 return sd;
 }
 
+static int setSocketNonBlocking(int sd, boolean set)
+/* Use socket control flags to set O_NONBLOCK if set==TRUE,
+ * or clear it if set==FALSE.
+ * Return -1 if there are any errors, 0 if successful. */
+{
+long fcntlFlags;
+// Set or clear non-blocking
+if ((fcntlFlags = fcntl(sd, F_GETFL, NULL)) < 0) 
+    {
+    warn("Error fcntl(..., F_GETFL) (%s)", strerror(errno));
+    return -1;
+    }
+if (set)
+    fcntlFlags |= O_NONBLOCK;
+else
+    fcntlFlags &= (~O_NONBLOCK);
+if (fcntl(sd, F_SETFL, fcntlFlags) < 0) 
+    {
+    warn("Error fcntl(..., F_SETFL) (%s)", strerror(errno));
+    return -1;
+    }
+return 0;
+}
+
+int setReadWriteTimeouts(int sd, int seconds)
+/* Set read and write timeouts on socket sd 
+ * Return -1 if there are any errors, 0 if successful. */
+{
+struct timeval timeout;      
+timeout.tv_sec = seconds;
+timeout.tv_usec = 0;
+
+if (setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout)) < 0)
+    {
+    warn("setsockopt failed setting socket receive timeout\n");
+    return -1;
+    }
+
+if (setsockopt(sd, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(timeout)) < 0)
+    {
+    warn("setsockopt failed setting socket send timeout\n");
+    return -1;
+    }
+return 0;
+}
+
+
+static struct timeval tvMinus(struct timeval a, struct timeval b)
+/* Return the result of a - b; this handles wrapping of milliseconds. 
+ * result.tv_usec will always be positive. 
+ * result.tv_sec will be negative if b > a. */
+{
+// subtract b from a.
+if (a.tv_usec < b.tv_usec)
+    {
+    a.tv_usec += 1000000;
+    a.tv_sec--;
+    }
+a.tv_usec -= b.tv_usec;
+a.tv_sec -= b.tv_sec;
+return a;
+}
+
 static int netConnectWithTimeout(char *hostName, int port, long msTimeout)
 /* In order to avoid a very long default timeout (several minutes) for hosts that will
  * not answer the port, we are forced to connect non-blocking.
- * After the connection has been established, we return to blocking mode. */
+ * After the connection has been established, we return to blocking mode.
+ * Also closes sd if error. */
 {
 int sd;
 struct sockaddr_in sai;		/* Some system socket info. */
 int res;
 fd_set mySet;
-struct timeval lTime;
-long fcntlFlags;
 
 if (hostName == NULL)
     {
@@ -61,17 +121,9 @@ if (!internetFillInAddress(hostName, port, &sai))
 if ((sd = netStreamSocket()) < 0)
     return sd;
 
-// Set non-blocking
-if ((fcntlFlags = fcntl(sd, F_GETFL, NULL)) < 0) 
+// Set socket to nonblocking so we can manage our own timeout time.
+if (setSocketNonBlocking(sd, TRUE) < 0)
     {
-    warn("Error fcntl(..., F_GETFL) (%s)", strerror(errno));
-    close(sd);
-    return -1;
-    }
-fcntlFlags |= O_NONBLOCK;
-if (fcntl(sd, F_SETFL, fcntlFlags) < 0) 
-    {
-    warn("Error fcntl(..., F_SETFL) (%s)", strerror(errno));
     close(sd);
     return -1;
     }
@@ -82,16 +134,35 @@ if (res < 0)
     {
     if (errno == EINPROGRESS)
 	{
+	struct timeval startTime;
+	gettimeofday(&startTime, NULL);
+	struct timeval remainingTime;
+	remainingTime.tv_sec = (long) (msTimeout/1000);
+	remainingTime.tv_usec = (long) (((msTimeout/1000)-remainingTime.tv_sec)*1000000);
 	while (1) 
 	    {
-	    lTime.tv_sec = (long) (msTimeout/1000);
-	    lTime.tv_usec = (long) (((msTimeout/1000)-lTime.tv_sec)*1000000);
 	    FD_ZERO(&mySet);
 	    FD_SET(sd, &mySet);
-	    res = select(sd+1, NULL, &mySet, &mySet, &lTime);
+	    // use tempTime (instead of using remainingTime directly) because on some platforms select() may modify the time val.
+	    struct timeval tempTime = remainingTime;
+	    res = select(sd+1, NULL, &mySet, &mySet, &tempTime);  
 	    if (res < 0) 
 		{
-		if (errno != EINTR) 
+		if (errno == EINTR)  // Ignore the interrupt 
+		    {
+                    // Subtract the elapsed time from remaining time since some platforms need this.
+		    struct timeval newTime;
+		    gettimeofday(&newTime, NULL);
+                    struct timeval elapsedTime = tvMinus(newTime, startTime);
+		    remainingTime = tvMinus(remainingTime, elapsedTime);
+		    if (remainingTime.tv_sec < 0)  // means our timeout has more than expired
+			{
+			remainingTime.tv_sec = 0;
+			remainingTime.tv_usec = 0;
+			}
+		    startTime = newTime;
+		    }
+		else
 		    {
 		    warn("Error in select() during TCP non-blocking connect %d - %s", errno, strerror(errno));
 		    close(sd);
@@ -137,16 +208,14 @@ if (res < 0)
     }
 
 // Set to blocking mode again
-if ((fcntlFlags = fcntl(sd, F_GETFL, NULL)) < 0)
+if (setSocketNonBlocking(sd, FALSE) < 0)
     {
-    warn("Error fcntl(..., F_GETFL) (%s)", strerror(errno));
     close(sd);
     return -1;
     }
-fcntlFlags &= (~O_NONBLOCK);
-if (fcntl(sd, F_SETFL, fcntlFlags) < 0)
+
+if (setReadWriteTimeouts(sd, DEFAULTREADWRITETTIMEOUTSEC) < 0)
     {
-    warn("Error fcntl(..., F_SETFL) (%s)", strerror(errno));
     close(sd);
     return -1;
     }
@@ -159,7 +228,7 @@ return sd;
 int netConnect(char *hostName, int port)
 /* Start connection with a server. */
 {
-return netConnectWithTimeout(hostName, port, 10000); // 10 seconds connect timeout
+return netConnectWithTimeout(hostName, port, DEFAULTCONNECTTIMEOUTMSEC); // 10 seconds connect timeout
 }
 
 int netMustConnect(char *hostName, int port)
@@ -279,12 +348,12 @@ if (!plumberInstalled)
     }
 }
 
-size_t netReadAll(int sd, void *vBuf, size_t size)
+ssize_t netReadAll(int sd, void *vBuf, ssize_t size)
 /* Read given number of bytes into buffer.
  * Don't give up on first read! */
 {
 char *buf = vBuf;
-size_t totalRead = 0;
+ssize_t totalRead = 0;
 int oneRead;
 
 if (!plumberInstalled)
@@ -301,11 +370,11 @@ while (totalRead < size)
 return totalRead;
 }
 
-int netMustReadAll(int sd, void *vBuf, size_t size)
+ssize_t netMustReadAll(int sd, void *vBuf, ssize_t size)
 /* Read given number of bytes into buffer or die.
  * Don't give up if first read is short! */
 {
-int ret = netReadAll(sd, vBuf, size);
+ssize_t ret = netReadAll(sd, vBuf, size);
 if (ret < 0)
     errnoAbort("Couldn't finish netReadAll");
 return ret;
@@ -376,6 +445,31 @@ if (x)
 
 }
 
+void cgiDecodeFull(char *in, char *out, int inLength)
+/* Out will be a cgi-decoded version of in (no space from plus!).
+ * Out will be a little shorter than in typically, and
+ * can be the same buffer. */
+{
+  char c;
+  int i;
+  for (i=0; i<inLength;++i)
+    {
+      c = *in++;
+      if (c == '%')
+	{
+          int code;
+          if (sscanf(in, "%2x", &code) != 1)
+	    code = '?';
+          in += 2;
+          i += 2;
+          *out++ = code;
+	}
+      else
+	*out++ = c;
+    }
+  *out++ = 0;
+}
+
 void netParseUrl(char *url, struct netParsedUrl *parsed)
 /* Parse a URL into components.   A full URL is made up as so:
  *   http://user:password@hostName:port/file;byterange=0-499
@@ -416,13 +510,28 @@ if (u == NULL)
     strcpy(parsed->file, "/");
 else
     {
+
     parseByteRange(u, &parsed->byteRangeStart, &parsed->byteRangeEnd, TRUE);
 
-    /* need to encode spaces, but not ! other characters */
-    char *t=replaceChars(u," ","%20");
-    safecpy(parsed->file, sizeof(parsed->file), t);
-    freeMem(t);
-    *u = 0;
+    if (sameWord(parsed->protocol,"http") ||
+        sameWord(parsed->protocol,"https"))
+	{
+	// http servers expect the URL request to be URL-encoded already.
+	/* need to encode spaces, but not ! other characters */
+	char *t=replaceChars(u," ","%20");
+	safecpy(parsed->file, sizeof(parsed->file), t);
+	freeMem(t);
+	}
+
+    *u = 0; // terminate the host:port string
+
+    if (sameWord(parsed->protocol,"ftp"))
+	{
+	++u; // that first slash is not considered part of the ftp path 
+	// decode now because the FTP server does NOT expect URL-encoding.
+	cgiDecodeFull(u,parsed->file,strlen(u));  // decodes %FF but not +
+	}
+
     }
 
 /* Split off user part */
@@ -531,10 +640,9 @@ if (npu->byteRangeStart != -1)
 return dyStringCannibalize(&dy);
 }
 
-/* this was cloned from rudp.c - move it later for sharing */
-static boolean readReadyWait(int sd, int microseconds)
-/* Wait for descriptor to have some data to read, up to
- * given number of microseconds. */
+int netWaitForData(int sd, int microseconds)
+/* Wait for descriptor to have some data to read, up to given number of
+ * microseconds.  Returns amount of data there or zero if timed out. */
 {
 struct timeval tv;
 fd_set set;
@@ -560,15 +668,23 @@ for (;;)
 	if (errno == EINTR)	/* Select interrupted, not timed out. */
 	    continue;
     	else 
-    	    warn("select failure in rudp: %s", strerror(errno));
+    	    warn("select failure %s", strerror(errno));
     	}
     else
 	{
-    	return readyCount > 0;	/* Zero readyCount indicates time out */
+    	return readyCount;	/* Zero readyCount indicates time out */
 	}
     }
 }
 
+static boolean readReadyWait(int sd, int microseconds)
+/* Wait for descriptor to have some data to read, up to given number of
+ * number of microseconds.  Returns true if there is data, false if timed out. */
+{
+int readyCount = netWaitForData(sd, microseconds);
+return readyCount > 0;	/* Zero readyCount indicates time out */
+}
+
 static void sendFtpCommandOnly(int sd, char *cmd)
 /* send command to ftp server */
 {   
@@ -1350,7 +1466,9 @@ while(TRUE)
 	    {
 	    mustUseProxyAuth = TRUE;
 	    }
-	else if (byteRangeUsed)
+	else if (byteRangeUsed 
+	    /* hack for Apache bug 2.2.20 and 2.2.21 2011-10-21 should be OK to remove after one year. */
+		&& !(sameString(code, "200") && byteRangeStart == 0 && byteRangeEnd == -1))  
 	    {
 	    if (!sameString(code, "206"))
 		{
@@ -1360,6 +1478,11 @@ while(TRUE)
 		return FALSE;
 		}
 	    }
+	else if (sameString(code, "404"))
+	    {
+	    warn("404 file not found on %s", url);
+	    return FALSE;
+	    }
 	else if (!sameString(code, "200"))
 	    {
 	    warn("Expected 200 %s: %s %s", url, code, line);
@@ -1413,7 +1536,9 @@ if (mustUseProxy ||  mustUseProxyAuth)
 	proxyLocation ? proxyLocation : "not given");
     return FALSE;
     }
-if (byteRangeUsed && !foundContentRange)
+if (byteRangeUsed && !foundContentRange
+	    /* hack for Apache bug 2.2.20 and 2.2.21 2011-10-21 should be OK to remove after one year. */
+		&& !(byteRangeStart == 0 && byteRangeEnd == -1))  
     {
     char bre[256];
     safef(bre, sizeof bre, "%lld", (long long)byteRangeEnd);
@@ -1556,9 +1681,14 @@ else
 	    url = newUrl;
 	    }
 	}
-    if (endsWith(url, ".gz") ||
-	endsWith(url, ".Z")  ||
-    	endsWith(url, ".bz2"))
+    char *urlDecoded = cloneString(url);
+    cgiDecode(url, urlDecoded, strlen(url));
+    boolean isCompressed =
+	(endsWith(urlDecoded,".gz") ||
+   	 endsWith(urlDecoded,".Z")  ||
+	 endsWith(urlDecoded,".bz2"));
+    freeMem(urlDecoded);
+    if (isCompressed)
 	{
 	lf = lineFileDecompressFd(url, TRUE, sd);
            /* url needed only for compress type determination */
@@ -1573,6 +1703,29 @@ else
     }
 }
 
+int netUrlMustOpenPastHeader(char *url)
+/* Get socket descriptor for URL.  Process header, handling any forwarding and
+ * the like.  Do errAbort if there's a problem, which includes anything but a 200
+ * return from http after forwarding. */
+{
+int sd = netUrlOpen(url);
+if (sd < 0)
+    noWarnAbort();
+int newSd = 0;
+if (startsWith("http://",url) || startsWith("https://",url))
+    {  
+    char *newUrl = NULL;
+    if (!netSkipHttpHeaderLinesHandlingRedirect(sd, url, &newSd, &newUrl))
+	noWarnAbort();
+    if (newUrl != NULL)
+	{
+	sd = newSd;
+	freeMem(newUrl); 
+	}
+    }
+return sd;
+}
+
 struct lineFile *netLineFileSilentOpen(char *url)
 /* Open a lineFile on a URL.  Just return NULL without any user
  * visible warning message if there's a problem. */
@@ -1596,652 +1749,6 @@ lineFileClose(&lf);
 return text;
 }
 
-
-struct parallelConn
-/* struct to information on a parallel connection */
-    {
-    struct parallelConn *next;  /* next connection */
-    int sd;                     /* socket descriptor */
-    off_t rangeStart;           /* where does the range start */
-    off_t partSize;             /* range size */
-    off_t received;             /* bytes received */
-    };
-
-void writeParaFetchStatus(char *origPath, struct parallelConn *pcList, char *url, off_t fileSize, char *dateString, boolean isFinal)
-/* Write a status file.
- * This has two purposes.
- * First, we can use it to resume a failed transfer.
- * Second, we can use it to follow progress */
-{
-char outTempX[1024];
-char outTemp[1024];
-safef(outTempX, sizeof(outTempX), "%s.paraFetchStatusX", origPath);
-safef(outTemp, sizeof(outTemp), "%s.paraFetchStatus", origPath);
-struct parallelConn *pc = NULL;
-
-FILE *f = mustOpen(outTempX, "w");
-int part = 0;
-fprintf(f, "%s\n", url);
-fprintf(f, "%lld\n", (long long)fileSize);
-fprintf(f, "%s\n", dateString);
-for(pc = pcList; pc; pc = pc->next)
-    {
-    fprintf(f, "part%d %lld %lld %lld\n", part
-	, (long long)pc->rangeStart
-	, (long long)pc->partSize
-	, (long long)pc->received);
-    ++part;
-    }
-
-carefulClose(&f);
-
-/* rename the successful status to the original name */
-rename(outTempX, outTemp);
-
-if (isFinal)  /* We are done and just looking to get rid of the file. */
-    unlink(outTemp);
-}
-
-
-boolean readParaFetchStatus(char *origPath, 
-    struct parallelConn **pPcList, char **pUrl, off_t *pFileSize, char **pDateString, off_t *pTotalDownloaded)
-/* Write a status file.
- * This has two purposes.
- * First, we can use it to resume a failed transfer.
- * Second, we can use it to follow progress */
-{
-char outTemp[1024];
-char outStat[1024];
-safef(outStat, sizeof(outStat), "%s.paraFetchStatus", origPath);
-safef(outTemp, sizeof(outTemp), "%s.paraFetch", origPath);
-struct parallelConn *pcList = NULL, *pc = NULL;
-off_t totalDownloaded = 0;
-
-if (!fileExists(outStat))
-    {
-    unlink(outTemp);
-    return FALSE;
-    }
-
-if (!fileExists(outTemp))
-    {
-    unlink(outStat);
-    return FALSE;
-    }
-
-char *line, *word;
-struct lineFile *lf = lineFileOpen(outStat, TRUE);
-if (!lineFileNext(lf, &line, NULL))
-    {
-    unlink(outTemp);
-    unlink(outStat);
-    return FALSE;
-    }
-char *url = cloneString(line);
-if (!lineFileNext(lf, &line, NULL))
-    {
-    unlink(outTemp);
-    unlink(outStat);
-    return FALSE;
-    }
-off_t fileSize = sqlLongLong(line);
-if (!lineFileNext(lf, &line, NULL))
-    {
-    unlink(outTemp);
-    unlink(outStat);
-    return FALSE;
-    }
-char *dateString = cloneString(line);
-while (lineFileNext(lf, &line, NULL))
-    {
-    word = nextWord(&line);
-    AllocVar(pc);
-    pc->next = NULL;
-    pc->sd = -4;  /* no connection tried yet */
-    word = nextWord(&line);
-    pc->rangeStart = sqlLongLong(word);
-    word = nextWord(&line);
-    pc->partSize = sqlLongLong(word);
-    word = nextWord(&line);
-    pc->received = sqlLongLong(word);
-    if (pc->received == pc->partSize)
-	pc->sd = -1;  /* part all done already */
-    totalDownloaded += pc->received;
-    slAddHead(&pcList, pc);
-    }
-slReverse(&pcList);
-
-lineFileClose(&lf);
-
-if (slCount(pcList) < 1)
-    {
-    unlink(outTemp);
-    unlink(outStat);
-    return FALSE;
-    }
-
-*pPcList = pcList;
-*pUrl = url;
-*pFileSize = fileSize;
-*pDateString = dateString;
-*pTotalDownloaded = totalDownloaded;
-
-return TRUE;
-
-}
-
-
-boolean parallelFetch(char *url, char *outPath, int numConnections, int numRetries, boolean newer, boolean progress)
-/* Open multiple parallel connections to URL to speed downloading */
-{
-char *origPath = outPath;
-char outTemp[1024];
-safef(outTemp, sizeof(outTemp), "%s.paraFetch", outPath);
-outPath = outTemp;
-/* get the size of the file to be downloaded */
-off_t fileSize = 0;
-off_t totalDownloaded = 0;
-ssize_t sinceLastStatus = 0;
-char *dateString = "";
-int star = 1;  
-int starMax = 20;  
-int starStep = 1;
-// TODO handle case-sensitivity of protocols input
-if (startsWith("http://",url) || startsWith("https://",url))
-    {
-    struct hash *hash = newHash(0);
-    int status = netUrlHead(url, hash);
-    if (status != 200) // && status != 302 && status != 301)
-	{
-	warn("Error code: %d, expected 200 for %s, can't proceed, sorry", status, url);
-	return FALSE;
-	}
-    char *sizeString = hashFindValUpperCase(hash, "Content-Length:");
-    if (sizeString)
-	{
-	fileSize = atoll(sizeString);
-	}
-    else
-	{
-	warn("No Content-Length: returned in header for %s, must limit to a single connection, will not know if data is complete", url);
-	numConnections = 1;
-	fileSize = -1;
-	}
-    char *ds = hashFindValUpperCase(hash, "Last-Modified:");
-    if (ds)
-	dateString = cloneString(ds);
-    hashFree(&hash);
-    }
-else if (startsWith("ftp://",url))
-    {
-    long long size = 0;
-    time_t t;
-    boolean ok = netGetFtpInfo(url, &size, &t);
-    if (!ok)
-	{
-	warn("Unable to get size info from FTP for %s, can't proceed, sorry", url);
-	return FALSE;
-	}
-    fileSize = size;
-
-    struct tm  *ts;
-    char ftpTime[80];
- 
-    /* Format the time "Tue, 15 Jun 2010 06:45:08 GMT" */
-    ts = localtime(&t);
-    strftime(ftpTime, sizeof(ftpTime), "%a, %d %b %Y %H:%M:%S %Z", ts);
-    dateString = cloneString(ftpTime);
-
-    }
-else
-    {
-    warn("unrecognized protocol: %s", url);
-    return FALSE;
-    }
-
-
-verbose(2,"fileSize=%lld\n", (long long) fileSize);
-
-if (fileSize < 65536)    /* special case small file */
-    numConnections = 1;
-
-if (numConnections > 50)    /* ignore high values for numConnections */
-    {
-    warn("Currently maximum number of connections is 50. You requested %d. Will proceed with 50 on %s", numConnections, url);
-    numConnections = 50;
-    }
-
-verbose(2,"numConnections=%d\n", numConnections); //debug
-
-if (numConnections < 1)
-    {
-    warn("number of connections must be greater than 0 for %s, can't proceed, sorry", url);
-    return FALSE;
-    }
-
-if (numRetries < 0)
-    numRetries = 0;
-
-/* what is the size of each part */
-off_t partSize = (fileSize + numConnections -1) / numConnections;
-if (fileSize == -1) 
-    partSize = -1;
-off_t base = 0;
-int c;
-
-verbose(2,"partSize=%lld\n", (long long) partSize); //debug
-
-
-/* n is the highest-numbered descriptor */
-int n = 0;
-int connOpen = 0;
-int reOpen = 0;
-
-
-struct parallelConn *restartPcList = NULL;
-char *restartUrl = NULL;
-off_t restartFileSize = 0;
-char *restartDateString = "";
-off_t restartTotalDownloaded = 0;
-boolean restartable = readParaFetchStatus(origPath, &restartPcList, &restartUrl, &restartFileSize, &restartDateString, &restartTotalDownloaded);
-if (fileSize == -1)
-    restartable = FALSE;
-
-struct parallelConn *pcList = NULL, *pc;
-
-if (restartable 
- && sameString(url, restartUrl)
- && fileSize == restartFileSize
- && sameString(dateString, restartDateString))
-    {
-    pcList = restartPcList;
-    totalDownloaded = restartTotalDownloaded;
-    }
-else
-    {
-
-    if (newer) // only download it if it is newer than what we already have
-	{
-	/* datestamp mtime from last-modified header */
-	struct tm tm;
-	// Last-Modified: Wed, 15 Nov 1995 04:58:08 GMT
-	// These strings are always GMT
-	if (strptime(dateString, "%a, %d %b %Y %H:%M:%S %Z", &tm) == NULL)
-	    {
-	    warn("unable to parse last-modified string [%s]", dateString);
-	    }
-	else
-	    {
-	    time_t t;
-	    // convert to UTC (GMT) time
-	    t = mktimeFromUtc(&tm);
-	    if (t == -1)
-		{
-		warn("mktimeFromUtc failed while converting last-modified string to UTC [%s]", dateString);
-		}
-	    else
-		{
-		// get the file mtime
-		struct stat mystat;
-		ZeroVar(&mystat);
-		if (stat(origPath,&mystat)==0)
-		    {
-		    if (t <= mystat.st_mtime)
-			{
-			verbose(2,"Since nothing newer was found, skipping %s\n", origPath);
-			verbose(3,"t from last-modified = %ld; st_mtime = %ld\n", (long) t, (long)mystat.st_mtime);
-			return TRUE;
-			}
-		    }
-		}
-	    }
-	}
-
-    /* make a list of connections */
-    for (c = 0; c < numConnections; ++c)
-	{
-	AllocVar(pc);
-	pc->next = NULL;
-	pc->rangeStart = base;
-	base += partSize;
-	pc->partSize = partSize;
-	if (fileSize != -1 && pc->rangeStart+pc->partSize >= fileSize)
-	    pc->partSize = fileSize - pc->rangeStart;
-	pc->received = 0;
-	pc->sd = -4;  /* no connection tried yet */
-	slAddHead(&pcList, pc);
-	}
-    slReverse(&pcList);
-    }
-
-if (progress)
-    {
-    char nicenumber[1024]="";
-    sprintWithGreekByte(nicenumber, sizeof(nicenumber), fileSize);
-    printf("downloading %s ", nicenumber); fflush(stdout);
-    starStep = fileSize/starMax;
-    if (starStep < 1)
-	starStep = 1;
-    }
-
-int out = open(outPath, O_CREAT|O_WRONLY, 0664);
-if (out < 0)
-    {
-    warn("Unable to open %s for write while downloading %s, can't proceed, sorry", url, outPath);
-    return FALSE;
-    }
-
-
-/* descriptors for select() */
-fd_set rfds;
-struct timeval tv;
-int retval;
-
-ssize_t readCount = 0;
-#define BUFSIZE 65536 * 4
-char buf[BUFSIZE];
-
-/* create paraFetchStatus right away for monitoring programs */
-writeParaFetchStatus(origPath, pcList, url, fileSize, dateString, FALSE);
-sinceLastStatus = 0;
-
-int retryCount = 0;
-
-time_t startTime = time(NULL);
-
-#define SELTIMEOUT 5
-#define RETRYSLEEPTIME 30    
-while (TRUE)
-    {
-
-    verbose(2,"Top of big loop\n");
-
-    if (progress)
-	{
-	while (totalDownloaded >= star * starStep)
-	    {
-	    printf("*");fflush(stdout);
-	    ++star;
-	    }
-	}
-
-    /* are we done? */
-    if (connOpen == 0)
-	{
-	boolean done = TRUE;
-	for(pc = pcList; pc; pc = pc->next)
-	    if (pc->sd != -1)
-		done = FALSE;
-	if (done) break;
-	}
-
-    /* See if we need to open any connections, either new or retries */
-    for(pc = pcList; pc; pc = pc->next)
-	{
-	if ((pc->sd == -4)  /* never even tried to open yet */
-	 || ((reOpen>0)      /* some connections have been released */
-	    && (pc->sd == -2  /* started but not finished */
-	    ||  pc->sd == -3)))  /* not even started */
-	    {
-	    char urlExt[1024];
-	    safef(urlExt, sizeof(urlExt), "%s;byterange=%llu-%llu"
-	    , url
-	    , (unsigned long long) (pc->rangeStart + pc->received)
-	    , (unsigned long long) (pc->rangeStart + pc->partSize - 1) );
-
-
-	    int oldSd = pc->sd;  /* in case we need to remember where we were */
-	    if (oldSd != -4)      /* decrement whether we succeed or not */
-		--reOpen;
-	    if (oldSd == -4) 
-		oldSd = -3;       /* ok this one just changes */
-	    if (fileSize == -1)
-		{
-		verbose(2,"opening url %s\n", url);
-		pc->sd = netUrlOpen(url);
-		}
-	    else
-		{
-		verbose(2,"opening url %s\n", urlExt);
-		pc->sd = netUrlOpen(urlExt);
-		}
-	    if (pc->sd < 0)
-		{
-		pc->sd = oldSd;  /* failed to open, can retry later */
-		}
-	    else
-		{
-		char *newUrl = NULL;
-		int newSd = 0;
-		if (startsWith("http://",url) || startsWith("https://",url))
-		    {
-		    if (!netSkipHttpHeaderLinesHandlingRedirect(pc->sd, urlExt, &newSd, &newUrl))
-			{
-			warn("Error processing http response for %s", url);
-			return FALSE;
-			}
-		    if (newUrl) 
-			{
-			/*  Update sd with newSd, replace it with newUrl, etc. */
-			pc->sd = newSd;
-			}
-		    }
-		++connOpen;
-		}
-	    }
-	}
-
-
-    if (connOpen == 0)
-	{
-	warn("Unable to open any connections to download %s, can't proceed, sorry", url);
-	return FALSE;
-	}
-
-
-    FD_ZERO(&rfds);
-    n = 0;
-    for(pc = pcList; pc; pc = pc->next)
-	{
-	if (pc->sd >= 0)
-	    {
-	    FD_SET(pc->sd, &rfds);    /* reset descriptor in readfds for select() */
-	    if (pc->sd > n)
-		n = pc->sd;
-	    }
-	}
-
-
-    /* Wait up to SELTIMEOUT seconds. */
-    tv.tv_sec = SELTIMEOUT;
-    tv.tv_usec = 0;
-    retval = select(n+1, &rfds, NULL, NULL, &tv);
-    /* Don't rely on the value of tv now! */
-
-    if (retval < 0)
-	{
-	perror("select retval < 0");
-	return FALSE;
-	}
-    else if (retval)
-	{
-
-	verbose(2,"returned from select, retval=%d\n", retval);
-
-	for(pc = pcList; pc; pc = pc->next)
-	    {
-	    if ((pc->sd >= 0) && FD_ISSET(pc->sd, &rfds))
-		{
-
-		verbose(2,"found a descriptor with data: %d\n", pc->sd);
-
-		readCount = read(pc->sd, buf, BUFSIZE);
-
-		verbose(2,"readCount = %lld\n", (long long) readCount);
-
-		if (readCount == 0)
-		    {
-		    close(pc->sd);
-
-		    verbose(2,"closing descriptor: %d\n", pc->sd);
-		    pc->sd = -1;
-
-		    if (fileSize != -1 && pc->received != pc->partSize)	
-			{
-			pc->sd = -2;  /* conn was closed before all data was sent, can retry later */
-			return FALSE;
-			}
-		    --connOpen;
-		    ++reOpen;
-		    writeParaFetchStatus(origPath, pcList, url, fileSize, dateString, FALSE);
-		    sinceLastStatus = 0;
-		    continue; 
-		    }
-		if (readCount < 0)
-		    {
-		    warn("error reading from socket for url %s", url);
-		    return FALSE;
-		    }
-
-		verbose(2,"rangeStart %llu  received %llu\n"
-			, (unsigned long long) pc->rangeStart
-			, (unsigned long long) pc->received );
-
-		verbose(2,"seeking to %llu\n", (unsigned long long) (pc->rangeStart + pc->received));
-
-		if (lseek(out, pc->rangeStart + pc->received, SEEK_SET) == -1)
-		    {
-		    perror("error seeking output file");
-		    warn("error seeking output file %s: rangeStart %llu  received %llu for url %s"
-			, outPath
-			, (unsigned long long) pc->rangeStart
-			, (unsigned long long) pc->received
-			, url);
-		    return FALSE;
-		    }
-		int writeCount = write(out, buf, readCount);
-		if (writeCount < 0)
-		    {
-		    warn("error writing output file %s", outPath);
-		    return FALSE;
-		    }
-		pc->received += readCount;
-		totalDownloaded += readCount;
-		sinceLastStatus += readCount;
-		if (sinceLastStatus >= 100*1024*1024)
-		    {
-		    writeParaFetchStatus(origPath, pcList, url, fileSize, dateString, FALSE);
-		    sinceLastStatus = 0;
-		    }
-		}
-	    }
-	}
-    else
-	{
-	warn("No data within %d seconds for %s", SELTIMEOUT, url);
-	/* Retry ? */
-	if (retryCount >= numRetries)
-	    {
-    	    return FALSE;
-	    }
-	else
-	    {
-	    ++retryCount;
-	    /* close any open connections */
-	    for(pc = pcList; pc; pc = pc->next)
-		{
-		if (pc->sd >= 0) 
-		    {
-		    close(pc->sd);
-		    verbose(2,"closing descriptor: %d\n", pc->sd);
-		    }
-		if (pc->sd != -1) 
-		    pc->sd = -4;
-		}
-	    connOpen = 0;
-	    reOpen = 0;
-	    /* sleep for a while, maybe the server will recover */
-	    sleep(RETRYSLEEPTIME);
-	    }
-	}
-
-    }
-
-close(out);
-
-/* delete the status file - by passing TRUE */
-writeParaFetchStatus(origPath, pcList, url, fileSize, dateString, TRUE); 
-
-/* restore original file datestamp mtime from last-modified header */
-struct tm tm;
-// Last-Modified: Wed, 15 Nov 1995 04:58:08 GMT
-// These strings are always GMT
-if (strptime(dateString, "%a, %d %b %Y %H:%M:%S %Z", &tm) == NULL)
-    {
-    warn("unable to parse last-modified string [%s]", dateString);
-    }
-else
-    {
-    time_t t;
-    // convert to UTC (GMT) time
-    t = mktimeFromUtc(&tm);
-    if (t == -1)
-	{
-	warn("mktimeFromUtc failed while converting last-modified string to UTC [%s]", dateString);
-	}
-    else
-	{
-	// update the file mtime
-	struct utimbuf ut;
-	struct stat mystat;
-	ZeroVar(&mystat);
-	if (stat(outTemp,&mystat)==0)
-	    {
-	    ut.actime = mystat.st_atime;
-	    ut.modtime = t;
-	    if (utime(outTemp, &ut)==-1)
-		{
-		char errMsg[256];
-                safef(errMsg, sizeof(errMsg), "paraFetch: error setting modification time of %s to %s\n", outTemp, dateString);
-		perror(errMsg);
-		}
-	    }
-	}
-    }
-
-/* rename the successful download to the original name */
-rename(outTemp, origPath);
-
-
-
-if (progress)
-    {
-    while (star <= starMax)
-	{
-	printf("*");fflush(stdout);
-	++star;
-	}
-    long timeDiff = (long)(time(NULL) - startTime);
-    if (timeDiff > 0)
-	{
-	printf(" %ld seconds", timeDiff);
-	float mbpersec =  ((totalDownloaded - restartTotalDownloaded)/1000000) / timeDiff;
-	printf(" %0.1f MB/sec", mbpersec);
-	}
-    printf("\n");fflush(stdout);
-    }
-
-if (fileSize != -1 && totalDownloaded != fileSize)
-    {
-    warn("Unexpected result: Total downloaded bytes %lld is not equal to fileSize %lld"
-	, (long long) totalDownloaded
-	, (long long) fileSize);
-    return FALSE;
-    }
-return TRUE;
-}
-
-
 struct lineFile *netLineFileOpen(char *url)
 /* Return a lineFile attached to url.  This one
  * will skip any headers.   Free this with
@@ -2589,6 +2096,11 @@ int netHttpGetMultiple(char *url, struct slName *queries, void *userData,
 
   return qCount;
 } /* netHttpMultipleQueries */
-
-
 #endif
+
+boolean hasProtocol(char *urlOrPath)
+/* Return TRUE if it looks like it has http://, ftp:// etc. */
+{
+return stringIn("://", urlOrPath) != NULL;
+}
+
